@@ -102,6 +102,8 @@ gvfs_backend_ftp_determine_features (GVfsFtpTask *task)
     { "EPRT", G_VFS_FTP_FEATURE_EPRT },
     { "EPSV", G_VFS_FTP_FEATURE_EPSV },
     { "UTF8", G_VFS_FTP_FEATURE_UTF8 },
+    { "AUTH TLS", G_VFS_FTP_FEATURE_AUTH_TLS },
+    { "AUTH SSL", G_VFS_FTP_FEATURE_AUTH_SSL },
   };
   guint i, j;
   char **reply;
@@ -139,6 +141,61 @@ gvfs_backend_ftp_determine_features (GVfsFtpTask *task)
   return TRUE;
 }
 
+static void
+gvfs_backend_ftp_determine_site_features (GVfsFtpTask *task)
+{
+  const struct {
+    const char *        name;        	/* name of feature */
+    GVfsFtpFeature      enable;         /* flags to enable with this feature */
+  } features[] = {
+    { "CHMOD", G_VFS_FTP_FEATURE_CHMOD },
+    { "CHGRP", G_VFS_FTP_FEATURE_CHGRP },
+  };
+  guint i, j;
+  char **reply;
+
+  if (g_vfs_ftp_task_is_in_error (task))
+    return;
+
+  if (!g_vfs_ftp_task_send_and_check (task, 0, NULL, NULL, &reply, "SITE HELP"))
+    {
+      g_vfs_ftp_task_clear_error (task);
+      return;
+    }
+
+  if (g_strv_length (reply) == 1)
+    {
+      /* vsftpd returns just a single string, so we split it into multiple
+       * and then treat it like a regular reply */
+      char **split;
+
+      split = g_strsplit (reply[0], " ", -1);
+      g_strfreev (reply);
+      reply = split;
+    }
+
+  for (i = 1; reply[i]; i++)
+    {
+      char *feature = reply[i];
+
+      while (g_ascii_isspace (feature[0]))
+        feature++;
+
+      for (j = 0; j < G_N_ELEMENTS (features); j++)
+        {
+          if (g_ascii_strcasecmp (feature, features[j].name) == 0)
+            {
+              g_debug ("# site feature %s supported\n", features[j].name);
+              task->backend->features |= 1 << features[j].enable;
+            }
+        }
+    }
+
+  g_strfreev (reply);
+
+  return;
+}
+
 static void
 gvfs_backend_ftp_determine_system (GVfsFtpTask *task)
 {
@@ -380,6 +437,7 @@ do_mount (GVfsBackend *backend,
     }
 
   addr = G_NETWORK_ADDRESS (ftp->addr);
+  g_object_ref (addr);
   port = g_network_address_get_port (addr);
   username = NULL;
   password = NULL;
@@ -504,6 +562,7 @@ try_login:
     }
   g_vfs_ftp_task_setup_connection (&task);
   gvfs_backend_ftp_determine_system (&task);
+  gvfs_backend_ftp_determine_site_features (&task);
   gvfs_backend_ftp_setup_directory_cache (ftp);
   gvfs_backend_ftp_determine_default_location (&task);
 
@@ -514,6 +573,7 @@ try_login:
    */
   if (!g_vfs_ftp_task_is_in_error (&task))
     {
+      g_object_unref (ftp->addr);
       ftp->addr = G_SOCKET_CONNECTABLE (g_vfs_ftp_connection_get_address (task.conn, &task.error));
       if (ftp->addr == NULL)
         {
@@ -1004,6 +1064,95 @@ do_query_info (GVfsBackend *backend,
   g_vfs_ftp_file_free (file);
 }
 
+static gboolean
+try_query_settable_attributes (GVfsBackend *backend,
+			       GVfsJobQueryAttributes *job,
+			       const char *filename)
+{
+  GVfsBackendFtp *ftp = G_VFS_BACKEND_FTP (backend);
+  GFileAttributeInfoList *list;
+
+  if (!g_vfs_backend_ftp_has_feature (ftp, G_VFS_FTP_FEATURE_CHMOD))
+    {    
+      g_vfs_job_failed (G_VFS_JOB (job),
+                        G_IO_ERROR,
+                        G_IO_ERROR_NOT_SUPPORTED,
+                        _("Operation unsupported"));
+      return FALSE;
+    }    
+
+  list = g_file_attribute_info_list_new ();
+
+  g_file_attribute_info_list_add (list,
+				  G_FILE_ATTRIBUTE_UNIX_MODE,
+				  G_FILE_ATTRIBUTE_TYPE_UINT32,
+				  G_FILE_ATTRIBUTE_INFO_COPY_WITH_FILE |
+				  G_FILE_ATTRIBUTE_INFO_COPY_WHEN_MOVED);
+  
+  g_vfs_job_query_attributes_set_list (job, list);
+  g_vfs_job_succeeded (G_VFS_JOB (job));
+  g_file_attribute_info_list_unref (list);
+  
+  return TRUE;
+}
+
+static void
+do_set_attribute (GVfsBackend *backend,
+                  GVfsJobSetAttribute *job,
+                  const char *filename,
+                  const char *attribute,
+                  GFileAttributeType type,
+                  gpointer value_p,
+                  GFileQueryInfoFlags flags)
+{
+  GVfsBackendFtp *ftp = G_VFS_BACKEND_FTP (backend);
+  GVfsFtpTask task = G_VFS_FTP_TASK_INIT (ftp, G_VFS_JOB (job));
+  GVfsFtpFile *file;
+
+  file = g_vfs_ftp_file_new_from_gvfs (ftp, filename);
+
+  if (strcmp (attribute, G_FILE_ATTRIBUTE_UNIX_MODE) == 0)
+    {
+      if (type != G_FILE_ATTRIBUTE_TYPE_UINT32) 
+        {
+          g_set_error_literal (&task.error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_INVALID_ARGUMENT,
+                               _("Invalid attribute type (uint32 expected)"));
+        }
+      else if (!g_vfs_backend_ftp_has_feature (ftp, G_VFS_FTP_FEATURE_CHMOD))
+        {
+          g_set_error_literal (&task.error,
+                               G_IO_ERROR,
+                               G_IO_ERROR_NOT_SUPPORTED,
+                               _("Operation unsupported"));
+        }
+      else
+        {
+          guint mode = *(guint32 *)value_p;
+
+          if (g_vfs_ftp_task_send (&task,
+                                   0,
+                                   "SITE CHMOD %04o %s",
+                                   mode,
+                                   g_vfs_ftp_file_get_ftp_path (file)))
+            {
+              g_vfs_ftp_dir_cache_purge_file (ftp->dir_cache, file);
+            }
+        }
+    }
+  else
+    {
+      g_set_error_literal (&task.error,
+                           G_IO_ERROR,
+                           G_IO_ERROR_NOT_SUPPORTED,
+                           _("Operation unsupported"));
+    }
+
+  g_vfs_ftp_task_done (&task);
+  g_vfs_ftp_file_free (file);
+}
+
 static void
 do_enumerate (GVfsBackend *backend,
               GVfsJobEnumerate *job,
@@ -1527,6 +1676,8 @@ g_vfs_backend_ftp_class_init (GVfsBackendFtpClass *klass)
   backend_class->delete = do_delete;
   backend_class->make_directory = do_make_directory;
   backend_class->move = do_move;
+  backend_class->try_query_settable_attributes = try_query_settable_attributes;
+  backend_class->set_attribute = do_set_attribute;
   backend_class->pull = do_pull;
 }
 
