@@ -270,6 +270,7 @@ _get_bluetooth_device_properties (const char *bdaddr, char **icon_name)
   return name;
 }
 
+#ifdef HAVE_HAL
 static gboolean
 _is_same_path (const char *path, int usb_bus_num, int usb_device_num, int usb_intf_num)
 {
@@ -323,22 +324,23 @@ _find_ods_usb_intfnum (DBusGProxy *obex_manager, int device_usb_bus_num, int dev
     }
   return -1;
 }
+#endif
 
 static gint
 _get_usb_intfnum_and_properties (DBusGProxy *obex_manager, const char *device, char **display_name, char **icon_name)
 {
-  char **obex_devices;
-  int num_obex_devices;
   int usb_bus_num;
   int usb_device_num;
   int usb_intf_num;
 #ifdef HAVE_HAL
+  char **obex_devices;
+  int num_obex_devices;
+  int n;
   DBusError dbus_error;
   DBusConnection *dbus_connection;
   LibHalContext *hal_ctx;
 #endif
   int ods_intf_num = 1;
-  int n;
 
   /* Parse the [usb:1,41,3] string */
   if (!g_str_has_prefix (device, "[usb:"))
@@ -502,9 +504,28 @@ g_vfs_backend_obexftp_finalize (GObject *object)
 static void
 g_vfs_backend_obexftp_init (GVfsBackendObexftp *backend)
 {
+  DBusConnection *conn;
+  DBusGConnection *connection;
+  DBusError error;
   GError *err = NULL;
 
-  backend->connection = dbus_g_bus_get (DBUS_BUS_SESSION, &err);
+  /* Otherwise dbus-glib doesn't setup it value types */
+  connection = dbus_g_bus_get (DBUS_BUS_SESSION, NULL);
+
+  if (connection != NULL)
+        dbus_g_connection_unref (connection);
+
+  /* Connect to the session bus via a private connection */
+  dbus_error_init (&error);
+  conn = dbus_bus_get_private (DBUS_BUS_SESSION, &error);
+  if (conn == NULL) {
+      g_printerr ("Connecting to session bus failed: %s\n", err->message);
+      dbus_error_free (&error);
+      return;
+  }
+  dbus_connection_setup_with_g_main (conn, NULL);
+
+  backend->connection = dbus_connection_get_g_connection (conn);
   if (backend->connection == NULL) {
       g_printerr ("Connecting to session bus failed: %s\n", err->message);
       g_error_free (err);
